# -*- coding: utf-8 -*-
import sys
try:
    # Python3  
    from urllib.parse import quote
except: 
    # Python2 
    from urllib import quote

# Python2和Python3兼容
PY3 = sys.version_info > (3, )

unicode = str if PY3 else unicode



# ----- coding:utf-8 ------
# 秘钥ID
APP_ID = '20190619000308802' #你的appid
SECRET_KEY = 'gAuBfGBhy7uCY37JtlYk' #你的密钥

# base api url
BASE_URL = 'api.fanyi.baidu.com'
API_URL = '/api/trans/vip/translate'

# 支持语言列表
LANGUAGES = ["auto", "zh", "en", "yue", "wyw", "jp", "kor", "fra", "spa", "th", "ara", "ru", "pt", "de", "it", "el", "nl", "pl", "bul", "est", "dan", "fin", "cs", "rom", "slo", "swe", "hu", "cht", "vie"]

# 状态码映射
STATU_CODE_MAP = {
    "52001": "1001",    # 请求超时
    "54000": "1002",    # 必填参数为空
}


# 随机数据最小和最大值，也可另设其它值
RANDOM_LOW = 32768
RANDOM_UP = 65536



# --*-- coding: utf-8 --*--

# 翻译器（翻译类型）
BAIDU = "baidu"
GOOGLE = "google"
# SOGOU = "sogou"
# YOUDAO = "youdao"

# 状态码和状态信息
SUCCESS_STATUE_CODE = "1000"                                # 成功
PARAM_STATUE_CODE = "1002"                                  # 必填参数为空
EXCEPT_STATU_CODE = "1003"                                  # 其它异常情况
INVALID_TRANS_TYPE = "INVALID TRANSLATE TYPE"               # 传入的翻译类型有误
INVALID_SOURCE_LANG = "INVALID SOURCE LANGUAGE"             # 传入的源语言参数有误
INVALID_DEST_LANG = "INVALID DESTINATION LANGUAGE"          # 传入的译文语言参数有误
INVALID_PARAMETER = "FROM_LANG OR TO_LANG OR QUERY_TEXT IS NONE"           # 传入的需翻译text有误

SUCCESS = "SUCCESS"

LANGMAP = {
    'zh': 'zh-cn',  # 中文
    'vie': 'vi',    # 越南语
    'cht': 'zh-tw', # 繁体中文
    'swe': 'sv',    # 瑞典语
    'rom': 'ro',    # 罗马尼亚语
    'fra': 'fr',    # 法语
    'jp': 'ja',     # 日语
    'kor': 'ko',    # 韩语
    'spa': 'es',    # 西班牙语
    'ara': 'ar',    # 阿拉伯语
    'bul': 'bg',    # 保加利亚语
    'est': 'et',    # 爱沙尼亚语
    'dan': 'da',    # 丹麦语
    'fin': 'fi',    # 芬兰语
    'sol': 'sl'     # 斯洛文尼亚语
}

# LANGMAP的key和value转换
LANGMAP_REVERSED = dict(map(reversed, LANGMAP.items()))


# -*- coding:utf-8 -*-
import json

from sangfor_trans.baidu_trans import baidu_api
from sangfor_trans.google_trans import client
from sangfor_trans.config import (
    GOOGLE, BAIDU, EXCEPT_STATU_CODE, INVALID_TRANS_TYPE, INVALID_PARAMETER, PARAM_STATUE_CODE
)


class GetTranslator(object):
    def __init__(self, trans_type):
        self.trans_type = trans_type

    def translate(self, from_lang, to_lang, query_text):
        trans_result = {}
        # 判读传入字段是否为空
        if not from_lang or not to_lang or not query_text:
            trans_result["statu_code"] = PARAM_STATUE_CODE
            trans_result["msg"] = INVALID_PARAMETER
            return json.dumps(trans_result)

        if not isinstance(query_text, list):
            query_text = query_text.replace("\n", "")

        if self.trans_type == BAIDU:
            translator = baidu_api.Translator()
        elif self.trans_type == GOOGLE:
            # translator = client.Translator(service_urls=["translate.google.cn"])
            translator = client.Translator()
        else:
            trans_result["statu_code"] = PARAM_STATUE_CODE
            trans_result["msg"] = INVALID_TRANS_TYPE
            return json.dumps(trans_result)

        trans_result = translator.translate(from_lang, to_lang, query_text)

        return trans_result
            



#/usr/bin/env python
# -*- coding:utf-8 -*-
import httplib
import md5
import urllib
import random
import traceback
import json

from sangfor_trans.baidu_trans.setting import (
    API_URL, APP_ID, BASE_URL, SECRET_KEY, LANGUAGES, STATU_CODE_MAP, RANDOM_LOW, RANDOM_UP
    )
from sangfor_trans.config import (
    SUCCESS_STATUE_CODE, EXCEPT_STATU_CODE, SUCCESS, INVALID_SOURCE_LANG, INVALID_DEST_LANG, 
    LANGMAP, LANGMAP_REVERSED
)


class Translator(object):
    """
    百度翻译操作类
    """
    def __init__(self):
       pass

    def translate(self, from_lang, to_lang, query_text):
        """
        百度翻译api入口
        from_lang:  源语言
        to_lang:    译文语言
        query_text: 翻译文本
        return：    返回翻译结果
        """
        from_lang = from_lang.lower().strip()
        to_lang = to_lang.lower().strip()
        if from_lang not in LANGUAGES:
            if from_lang in LANGMAP_REVERSED:
                from_lang = LANGMAP_REVERSED[from_lang]
            else:
                return json.dumps({'msg': INVALID_SOURCE_LANG, 'statu_code': EXCEPT_STATU_CODE})

        if to_lang not in LANGUAGES:
            if to_lang in LANGMAP_REVERSED:
                to_lang = LANGMAP_REVERSED[to_lang]
            else:
                return json.dumps({'msg': INVALID_DEST_LANG, 'statu_code': EXCEPT_STATU_CODE})

        httpClient = None
        url = self.get_url(from_lang, to_lang, query_text)
        try:
            # API HTTP请求
            httpClient = httplib.HTTPConnection(BASE_URL)
            httpClient.request('GET', url)
        
            # 创建HTTPResponse对象
            response = httpClient.getresponse()
            resp = response.read()

            result = json.loads(resp)
            # 错误码和错误信息处理
            if "error_code" not in result:
                result["statu_code"] = SUCCESS_STATUE_CODE
                result["msg"] = SUCCESS
            else:
                if result["error_code"] in STATU_CODE_MAP:
                    result["statu_code"] = STATU_CODE_MAP.get(result["error_code"])
                else:
                    result["statu_code"] = EXCEPT_STATU_CODE
                result["msg"] = result["error_msg"]
                del result["error_code"]
                del result["error_msg"]
            
            return json.dumps(result)
             
        except Exception as e:
            traceback.print_exc()
        finally:
            if httpClient:
                httpClient.close()

    @staticmethod
    def get_url(from_lang, to_lang, query_text):
        """
        生成请求url
        """
        # 随机生成数据
        salt = random.randint(RANDOM_LOW, RANDOM_UP)
        # MD5生成签名
        sign = APP_ID + query_text + str(salt) + SECRET_KEY
        m1 = md5.new()
        m1.update(sign)
        sign = m1.hexdigest()
        # 拼接URL
        url = API_URL +'?appid=' + APP_ID + '&q=' + urllib.quote(query_text) + '&from=' + from_lang + '&to=' + to_lang + '&salt=' + str(salt) + '&sign=' + sign
        return url




"""Free Google Translate API for Python. Translates totally free of charge."""
# __all__ = 'Translator',
# __version__ = '2.4.0'


# from google_trans.client import Translator
# from google_trans.constants import LANGCODES, LANGUAGES



# -*- coding: utf-8 -*-
"""
谷歌翻译模块，可以通过该模块进行翻译.
"""
import requests
import random
import json
import traceback

from sangfor_trans.google_trans import urls, utils
from sangfor_trans.google_trans.adapters import TimeoutAdapter
from sangfor_trans.google_trans.compat import PY3
from sangfor_trans.google_trans.gtoken import TokenAcquirer
from sangfor_trans.google_trans.constants import DEFAULT_USER_AGENT, LANGCODES, LANGUAGES, SPECIAL_CASES
from sangfor_trans.google_trans.models import Translated, Detected
from sangfor_trans.config import (
    SUCCESS, SUCCESS_STATUE_CODE, EXCEPT_STATU_CODE, INVALID_DEST_LANG, 
    INVALID_SOURCE_LANG, PARAM_STATUE_CODE, INVALID_PARAMETER, LANGMAP
)


EXCLUDES = ('en', 'ca', 'fr')


class Translator(object):
    """
    谷歌翻译操作类，初始化、获取服务翻译url、翻译内容等
    """
    def __init__(self, service_urls=None, user_agent=DEFAULT_USER_AGENT, proxies=None, timeout=None):
        """
        创建会话对象，处理请求头，模拟浏览器访问
        """
        self.session = requests.Session()
        if proxies:
            self.session.proxies = proxies
        self.session.headers.update({
            'User-Agent': user_agent,
        })

        if timeout:
            self.session.mount('https://', TimeoutAdapter(timeout))
            self.session.mount('http://', TimeoutAdapter(timeout))

        self.service_urls = service_urls or [urls.SERVICE_URL]
        self.token_acquirer = TokenAcquirer(session=self.session, host=self.service_urls[0])

    def _pick_service_url(self):
        """
        获取翻译服务url
        """
        if len(self.service_urls) == 1:
            return self.service_urls[0]
        return random.choice(self.service_urls)

    def _translate(self, from_lang, to_lang, query_text ):
        """

        """
        if not PY3 and isinstance(query_text, str):  
            query_text = query_text.decode('utf-8')

        token = self.token_acquirer.do(query_text)
        params = utils.build_params(query=query_text, src=from_lang, dest=to_lang, token=token)
        url = urls.TRANSLATE.format(host=self._pick_service_url())
        r = self.session.get(url, params=params)

        data = utils.format_json(r.text)
        return data

    def _parse_extra_data(self, data):
        response_parts_name_mapping = {
            0: 'translation',
            1: 'all-translations',
            2: 'original-language',
            5: 'possible-translations',
            6: 'confidence',
            7: 'possible-mistakes',
            8: 'language',
            11: 'synonyms',
            12: 'definitions',
            13: 'examples',
            14: 'see-also',
        }
        extra = {}
        for index, category in response_parts_name_mapping.items():
            extra[category] = data[index] if (index < len(data) and data[index]) else None
        return extra

    def translate(self, from_lang, to_lang, query_text):
        """
        参数校验、翻译内容判断、格式处理、进行翻译
        return: 返回翻译结果，json格式
        """
        to_lang = to_lang.lower().split('_', 1)[0]
        from_lang = from_lang.lower().split('_', 1)[0]

        if from_lang != 'auto' and from_lang not in LANGUAGES:
            if from_lang in SPECIAL_CASES:
                from_lang = SPECIAL_CASES[from_lang]
            elif from_lang in LANGCODES:
                from_lang = LANGCODES[from_lang]
            elif from_lang in LANGMAP:
                from_lang = LANGMAP[from_lang]
            else:
                return json.dumps({'msg': INVALID_SOURCE_LANG, "statu_code": EXCEPT_STATU_CODE})

        if to_lang not in LANGUAGES:
            if to_lang in SPECIAL_CASES:
                to_lang = SPECIAL_CASES[to_lang]
            elif to_lang in LANGCODES:
                to_lang = LANGCODES[to_lang]
            elif to_lang in LANGMAP:
                to_lang = LANGMAP[to_lang]
            else:
                return json.dumps({'msg': INVALID_DEST_LANG, "statu_code": EXCEPT_STATU_CODE})

        # 翻译内容为list，则走这里进行处理
        if isinstance(query_text, list):
            result = []
            for item in query_text:
                translated = self.translate(from_lang, to_lang, item)
                result.append(translated)
            return result

        origin = query_text
        data = self._translate(from_lang, to_lang, query_text)

        # 格式处理
        translated = ''.join([d[0] if d[0] else '' for d in data[0]])

        extra_data = self._parse_extra_data(data)

        # 当from_lang=auto时，Translator将识别的实际源语言
        try:
            from_lang = data[2]
        except Exception:  
            traceback.print_exc()

        pron = origin
        try:
            pron = data[0][1][-2]
        except Exception:  
            traceback.print_exc()
        if not PY3 and isinstance(pron, unicode) and isinstance(origin, str):  
            origin = origin.decode('utf-8')
        if to_lang in EXCLUDES and pron == origin:
            pron = translated

        # 兼容Python2
        if not PY3:  
            if isinstance(from_lang, str):
                from_lang = from_lang.decode('utf-8')
            if isinstance(to_lang, str):
                to_lang = to_lang.decode('utf-8')
            if isinstance(translated, str):
                translated = translated.decode('utf-8')

        # 最后结果值处理输出成Translated对象格式
        result = Translated(src=from_lang, dest=to_lang, origin=origin,
                            text=translated, pronunciation=pron, extra_data=extra_data).text
        if result:
            ret_list = [{u'src': query_text.decode("utf-8"), u'dst': result}]
            trans_result = {'msg': SUCCESS, 'statu_code': SUCCESS_STATUE_CODE, 'from': from_lang, 'to': to_lang, 'trans_result': ret_list}
        else:
            trans_result = {'msg': INVALID_QUERY_TEXT, 'statu_code': PARAM_STATUE_CODE}
        return json.dumps(trans_result)

    def detect(self, query_text):
        """
        检测翻译内容属于哪种语言
        """
        if isinstance(query_text, list):
            result = []
            for item in query_text:
                lang = self.detect(item)
                result.append(lang)
            return result

        data = self._translate(from_lang='auto', to_lang='en', query_text=query_text)

        from_lang = ''
        confidence = 0.0
        try:
            from_lang = ''.join(data[8][0])
            confidence = data[8][-2][0]
        except Exception:  
            trackback.print_exc()
        result = Detected(lang=from_lang, confidence=confidence)

        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os.path
import re
import sys

from setuptools import setup, find_packages


def get_file(*paths):
    path = os.path.join(*paths)
    try:
        with open(path, 'rb') as f:
            return f.read().decode('utf8')
    except IOError:
        pass


def get_version():
    init_py = get_file(os.path.dirname(__file__), 'sangfor_trans', '__init__.py')
    pattern = r"{0}\W*=\W*'([^']+)'".format('__version__')
    version, = re.findall(pattern, init_py)
    return version


def get_description():
    init_py = get_file(os.path.dirname(__file__), 'sangfor_trans', '__init__.py')
    pattern = r'"""(.*?)"""'
    description, = re.findall(pattern, init_py, re.DOTALL)
    return description


def get_readme():
    # return "This is  Sangfor translation third-party library."
    return get_file(os.path.dirname(__file__), 'README.md')


def install():
    setup(
        name='sangfor_trans',
        version=get_version(),
        description=get_description(),
        long_description=get_readme(),
        long_description_content_type="text/markdown",
        license='MIT',
        author='scoefield',
        author_email='scoefield' '@' '163.com',
        url='https://github.com/Scoefield/SangforTrans',
        classifiers=['Development Status :: 5 - Production/Stable',
                     'Intended Audience :: Education',
                     'Intended Audience :: End Users/Desktop',
                     'License :: Freeware',
                     'Operating System :: POSIX',
                     'Operating System :: Microsoft :: Windows',
                     'Operating System :: MacOS :: MacOS X',
                     'Topic :: Education',
                     'Programming Language :: Python :: 2.7',
                     'Programming Language :: Python :: 3.4',
                     'Programming Language :: Python :: 3.5',
                     'Programming Language :: Python :: 3.6'],
        packages=find_packages(exclude=['docs']),
        keywords='sangfor translate translator',
    )


if __name__ == "__main__":
    install()
